from prsmsp.models.auth import APIKeyAuth, UnamePassAuth


class AuthFactory:
    """AuthFactory."""

    @staticmethod
    def get(auth_type: str):
        """Static method to get the class of authentication.

        :param auth_type:
        :type auth_type: str
        """
        auth_types = {
            "api_key": APIKeyAuth,
            "username_pass": UnamePassAuth,
        }

        Auth = auth_types.get(auth_type, None)

        if Auth is None:
            # if the auth method is not supported
            raise ValueError

        # this will not initiate the object, its just returning.
        # if you write like Auth() it will be initiated.
        # this will avoid the Input Error Execption.
        return Auth



import sys
import json

import requests

from prsmsp.abstracts.abcpanel import ABCSmsPanel
from prsmsp.factories import AuthFactory
from prsmsp.models import Response


class Mediana(ABCSmsPanel):
    # this is the client version for mediana package
    # this is not relevant to our package version,
    # its just the version that mediana package uses.
    __client_version = "1.0.1"
    __user_agent = (
        f"MedianaSMS/ApiClient/{__client_version} Python/{str(sys.hexversion)}"
    )

    def __init__(self, api_key: str) -> None:
        """Take the auth info

        :param api_key: your mediana api key auth
        :type api_key: str

        :rtype None
        :return: None
        """
        self.auth = AuthFactory.get("api_key")(api_key)

    def _response_parser(self, resp: requests.Response) -> Response:
        status_code = int(resp.status_code)
        real_response = json.loads(resp.text)

        return Response(status_code, real_response)

    def send_sms(self, receptor: str, message: str, originator: str) -> Response:
        """send sms with mediana sms panel

        :param receptor: reciver of your message
        :type receptor: str

        :param message: the message you want to send
        :type message: str

        :rtype Response
        :return: The requests response
        """

        url = "http://rest.medianasms.com/v1/messages"  # the url for send sms

        params = {
            "originator": originator,
            "recipients": [receptor],
            "message": message,
        }

        headers = requests.utils.default_headers()
        # this is just a simple defualt header.
        # mediana require this, also they implement it in this way.
        auth_header = f"AccessKey {self.auth.api_key}"
        headers.update(
            {
                "Content-Type": "application/json",
                "Accept": "application/json",
                "Authorization": auth_header,
                "User-Agent": self.__user_agent,
            }
        )

        resp = requests.get(url, headers=headers, params=params)

        return self._response_parser(resp)



import json

import requests

from prsmsp.abstracts.abcpanel import ABCSmsPanel
from prsmsp.factories import AuthFactory
from prsmsp.models import Response


class Kavenegar(ABCSmsPanel):
    def __init__(self, api_key: str) -> None:
        """Take the auth info

        :param api_key: your kavenegar api key auth
        :type api_key: str

        :rtype None
        :return: None
        """
        self.auth = AuthFactory.get("api_key")(api_key)

    def _response_parser(self, resp: requests.Response) -> Response:
        status_code = int(resp.status_code)
        real_response = json.loads(resp.text)

        return Response(status_code, real_response)

    def send_sms(self, receptor: str, message: str) -> Response:
        """send sms with kavenegar sms panel

        :param receptor: reciver of your message
        :type receptor: str

        :param message: the message you want to send
        :type message: str

        :rtype Response
        :return: The requests response
        """

        url = f"https://api.kavenegar.com/v1/{self.auth.api_key}/sms/send.json"

        params = {
            "receptor": receptor,
            "message": message,
        }

        resp = requests.get(url, params=params)

        return self._response_parser(resp)



class AuthTypeError(Exception):
    """AuthTypeError."""

    pass


class AuthInfoError(Exception):
    """AuthInfoError."""

    pass


class StatusCodeUnvalid(Exception):
    """StatusCodeUnvalid."""

    pass


class ResponseError(Exception):
    """ResponseError."""

    pass



import json

import requests

from prsmsp.abstracts.abcpanel import ABCSmsPanel
from prsmsp.factories import AuthFactory
from prsmsp.models import Response


class SmsDotIr(ABCSmsPanel):
    def __init__(self, api_key: str) -> None:
        """Take the auth info

        :param api_key: your smsir api key auth
        :type api_key: str

        :rtype None
        :return: None
        """
        self.auth = AuthFactory.get("api_key")(api_key)

    def _response_parser(self, resp: requests.Response) -> Response:
        status_code = int(resp.status_code)
        real_response = json.loads(resp.text)

        return Response(status_code, real_response)

    def send_sms(
        self, receptor: str, message: str, originator: str
    ) -> Response:  # NoQA
        """send sms with smsir sms panel

        :param receptor: reciver of your message
        :type receptor: str

        :param message: the message you want to send
        :type message: str

        :param originator: the originator that you want to send your message
        :type originator: str

        :rtype Response
        :return: The requests response
        """
        receptors = []
        messages = []

        headers = {
            "X-API-KEY": self.auth.api_key,
            "ACCEPT": "application/json",
            "Content-Type": "application/json",
        }

        url = "https://api.sms.ir/v1/send/likeToLike"

        receptors.append(receptor)
        messages.append(message)

        data = {
            "LineNumber": originator,
            "MessageTexts": messages,
            "Mobiles": receptors,
        }

        resp = requests.post(url, headers=headers, json=data)

        return self._response_parser(resp)



import json

import requests

from prsmsp.abstracts.abcpanel import ABCSmsPanel
from prsmsp.factories import AuthFactory
from prsmsp.models import Response


class GhasedakSms(ABCSmsPanel):
    def __init__(self, api_key: str) -> None:
        """Take the auth info

        :param api_key: your ghasedak sms api key auth
        :type api_key: str

        :rtype None
        :return: None
        """
        self.auth = AuthFactory.get("api_key")(api_key)

    def _response_parser(self, resp: requests.Response) -> Response:
        status_code = int(resp.status_code)
        real_response = json.loads(resp.text)

        return Response(status_code, real_response)

    def send_sms(self, receptor: str, message: str, originator: str) -> Response:
        """send sms with ghasedak sms panel

        :param receptor: reciver of your message
        :type receptor: str

        :param message: the message you want to send
        :type message: str

        :param originator: the originator that you want to send your message
        :type originator: str

        :rtype Response
        :return: The requests response
        """

        url = "https://api.ghasedak.me/v2/sms/send/simple"

        data = {
            "receptor": receptor,
            "message": message,
            "originator": originator,
        }

        headers = {
            "Accept": "application/json",
            "Content-Type": "application/x-www-form-urlencoded",
            "charset": "utf-8",
            "apikey": self.auth.api_key,
        }

        resp = requests.post(url, data=data, headers=headers)

        return self._response_parser(resp)



import abc


class ABCSmsPanel(abc.ABC):
    """ABCSmsPanel."""

    @abc.abstractmethod
    def _response_parser(self):
        raise NotImplementedError

    @abc.abstractmethod
    def send_sms(self):
        raise NotImplementedError
